@@ -18,7 +18,7 @@ if ($handle->{rc} != $STAF::kOk) {
 }
 my $result1 = send_request($handle, "SERVICE",
                            "ADD SERVICE FirstTest LIBRARY $dll EXECUTE SimpleService OPTION USELIB=\"$uselib\" PARMS=\"die\"",
-                           27, undef);
+                           27, "6:Error constructing service");
 my $result2 = send_request($handle, "FirstTest", "Ping", 2, "FirstTest");
 my $result3 = send_request($handle, "Ping", "Ping", 0, "PONG");
 print "Test - ", ($result1 and $result2 and $result3 ? "OK" : "NOT OK"), "\n";
